@@ -31,7 +31,9 @@
 #' Requires the qvalue Bioconductor package.
 #' @param ... Other arguments passed to [easylabel()].
 #' @seealso [easylabel()] [easyMAplot()]
-#' @return No return value
+#' @return By default no return value. If `output_shiny = FALSE` or the shiny 
+#' button 'Export plotly & exit' is pressed, a plotly figure is returned. 
+#' See [easylabel()].
 #' @importFrom stats p.adjust
 #' @export
 
@@ -185,12 +187,14 @@ easyVolcano <- function(data, x = NULL, y = NULL, padj = y,
 #' Requires the qvalue Bioconductor package.
 #' @param ... Other arguments passed to [easylabel()].
 #' @seealso [easylabel()] [easyVolcano()]
-#' @return No return value
+#' @return By default no return value. If `output_shiny = FALSE` or the shiny 
+#' button 'Export plotly & exit' is pressed, a plotly figure is returned. 
+#' See [easylabel()].
 #' @importFrom stats p.adjust
 #' @export
 
 
-easyMAplot <- function(data, x = NULL, y = NULL, padj = y, fdrcutoff = 0.05,
+easyMAplot <- function(data, x = NULL, y = NULL, padj = NULL, fdrcutoff = 0.05,
                        colScheme = c('darkgrey', 'blue', 'red'),
                        hline = 0,
                        labelDir = 'yellipse',
@@ -216,7 +220,7 @@ easyMAplot <- function(data, x = NULL, y = NULL, padj = y, fdrcutoff = 0.05,
   }
   if (is.null(x)) {
     if ('baseMean' %in% colnames(data)) {
-      data[, 'logmean'] <- log2(data[, 'baseMean'])  # DESeq2
+      data[, 'logmean'] <- log2(data[, 'baseMean'] + 0.125)  # DESeq2
       x <- 'logmean'
     }
     if ('AveExpr' %in% colnames(data))
@@ -253,7 +257,7 @@ easyMAplot <- function(data, x = NULL, y = NULL, padj = y, fdrcutoff = 0.05,
   } else custom_annotation = NULL
   
   # if using nominal p values
-  fdr_or_p <- if (y == padj) "P<" else "FDR<"
+  fdr_or_p <- if (padj %in% c('pvalue', 'P.Value')) "P<" else "FDR<"
   if (!(length(colScheme) - 1) %in% (length(fdrcutoff) * 1:2)) {
     stop("Number of colours in 'colScheme' does not fit with number of cuts in 'fdrcut'")
   }
@@ -293,6 +297,9 @@ easyMAplot <- function(data, x = NULL, y = NULL, padj = y, fdrcutoff = 0.05,
 #' @param pos The column of SNP positions in `data`.
 #' @param p The column of p values in `data`.
 #' @param labs The column of labels in `data`.
+#' @param startLabels Vector of initial labels. With a character vector, labels 
+#' are identified in the column specified by `labs`. With a numeric vector,
+#' points to be labelled are referred to by row number.
 #' @param pcutoff Cut-off for p value significance. Defaults to 5E-08.
 #' @param chromGap Size of gap between chromosomes along the x axis in base 
 #' pairs. If `NULL` this is automatically calculated dependent on the size of 
@@ -325,13 +332,16 @@ easyMAplot <- function(data, x = NULL, y = NULL, padj = y, fdrcutoff = 0.05,
 #' @param filename Filename for saving to pdf.
 #' @param ... Other arguments passed to [easylabel()].
 #' @seealso [easylabel()] [easyVolcano()]
-#' @return No return value
+#' @return By default no return value. If `output_shiny = FALSE` or the shiny 
+#' button 'Export plotly & exit' is pressed, a plotly figure is returned. 
+#' See [easylabel()].
 #' @importFrom gtools mixedsort
 #' @importFrom splus2R peaks
 #' @export
 
 easyManhattan <- function(data, chrom = 'chrom', pos = 'pos', p = 'p',
                           labs = 'rsid',
+                          startLabels = NULL,
                           pcutoff = 5e-08,
                           chromGap = NULL,
                           chromCols = c('royalblue', 'skyblue'),
@@ -404,6 +414,11 @@ easyManhattan <- function(data, chrom = 'chrom', pos = 'pos', p = 'p',
                    labels = levels(data[, chrom]))
   } else xticks <- NULL
   
+  labelchoices <- if (is.null(labs)) rownames(data) else data[[labs]]
+  if (is.character(startLabels)) {
+    startLabels <- which(data[[labs]] %in% startLabels)
+  } 
+  
   # find local maxima
   if (!is.null(npeaks)) {
     cat("Finding peaks...\n")
@@ -412,16 +427,20 @@ easyManhattan <- function(data, chrom = 'chrom', pos = 'pos', p = 'p',
     pks_index <- which(pks)
     sort_pks <- pks_index[order(data$logP[pks_index], decreasing = TRUE, 
                                 na.last = NA)]
-    startLabels <- sort_pks[1:npeaks]
-  } else startLabels <- NULL
+    rsLabels <- c(startLabels, sort_pks[1:npeaks])
+  } else {rsLabels <- startLabels}
+  
+  # fix x axis for locus plots
+  if(length(unique(data$chrom)) == 1) data$genome_pos <- data$pos
   
   if (!transpose) {
     easylabel(data, x = 'genome_pos', y = 'logP',
             labs = labs,
-            xlab = xlab, ylab = ylab,
+            xlab = xlab, 
+            ylab = ylab,
             xticks = xticks,
             labelDir = labelDir,
-            startLabels = startLabels,
+            startLabels = rsLabels,
             col = 'col', colScheme = colScheme, alpha = alpha,
             outline_col = outline_col,
             shapeScheme = shapeScheme,
